-- fsm.vhd: Finite State Machine
-- Author(s): Jan Kaláb (xkalab00)
--
library ieee;
use ieee.std_logic_1164.all;
-- ----------------------------------------------------------------------------
--                        Entity declaration
-- ----------------------------------------------------------------------------
entity fsm is
port(
   CLK         : in  std_logic;
   RESET       : in  std_logic;

   -- Input signals
   KEY         : in  std_logic_vector(15 downto 0);
   CNT_OF      : in  std_logic;

   -- Output signals
   FSM_CNT_CE  : out std_logic;
   FSM_MX_MEM  : out std_logic;
   FSM_MX_LCD  : out std_logic;
   FSM_LCD_WR  : out std_logic;
   FSM_LCD_CLR : out std_logic
);
end entity fsm;

-- ----------------------------------------------------------------------------
--                      Architecture declaration
-- ----------------------------------------------------------------------------
architecture behavioral of fsm is
   type t_state is (IDLE, OK1, OK2, OK3, OK4, OK5, OK6, OK7, OK8, OK9, PRINT_MESS_OK, PRINT_MESS_BAD, FINISH, BAD);
   signal present_state, next_state : t_state;

begin
-- -------------------------------------------------------
sync_logic : process(RESET, CLK)
begin
   if (RESET = '1') then
      present_state <= IDLE;
   elsif (CLK'event AND CLK = '1') then
      present_state <= next_state;
   end if;
end process sync_logic;

-- -------------------------------------------------------
next_state_logic : process(present_state, KEY, CNT_OF)
begin
   case (present_state) is
   -- - - - - - - - - - - - - - - - - - - - - - -
   when IDLE =>
      if (KEY(15 downto 0) = "0000000000000000") then -- nic
          next_state <= IDLE;
	  elsif (KEY(3) = '1') then -- 3
	  	  next_state <= OK1;
	  elsif (KEY(15) = '1') then -- #
	  	  next_state <= PRINT_MESS_BAD;
	  else next_state <= BAD; -- jiny
	  end if;
   -- - - - - - - - - - - - - - - - - - - - - - -
   when OK1 =>
      if (KEY(15 downto 0) = "0000000000000000") then -- nic
          next_state <= OK1;
	  elsif (KEY(6) = '1') then -- 6
	  	  next_state <= OK2;
	  elsif (KEY(15) = '1') then -- #
	  	  next_state <= PRINT_MESS_BAD;
	  else next_state <= BAD; -- jiny
	  end if;
   -- - - - - - - - - - - - - - - - - - - - - - -
   when OK2 =>
      if (KEY(15 downto 0) = "0000000000000000") then -- nic
          next_state <= OK2;
	  elsif (KEY(5) = '1') then -- 5
	  	  next_state <= OK3;
	  elsif (KEY(15) = '1') then -- #
	  	  next_state <= PRINT_MESS_BAD;
	  else next_state <= BAD; -- jiny
	  end if;
   -- - - - - - - - - - - - - - - - - - - - - - -
   when OK3 =>
      if (KEY(15 downto 0) = "0000000000000000") then -- nic
          next_state <= OK3;
	  elsif (KEY(9) = '1') then -- 9
	  	  next_state <= OK4;
	  elsif (KEY(15) = '1') then -- #
	  	  next_state <= PRINT_MESS_BAD;
	  else next_state <= BAD; -- jiny
	  end if;
   -- - - - - - - - - - - - - - - - - - - - - - -
   when OK4 =>
      if (KEY(15 downto 0) = "0000000000000000") then -- nic
          next_state <= OK4;
	  elsif (KEY(7) = '5') then -- 5
	  	  next_state <= OK5;
	  elsif (KEY(15) = '1') then -- #
	  	  next_state <= PRINT_MESS_BAD;
	  else next_state <= BAD; -- jiny
	  end if;
   -- - - - - - - - - - - - - - - - - - - - - - -
   when OK5 =>
      if (KEY(15 downto 0) = "0000000000000000") then -- nic
          next_state <= OK5;
	  elsif (KEY(7) = '1') then -- 7
	  	  next_state <= OK6;
	  elsif (KEY(15) = '1') then -- #
	  	  next_state <= PRINT_MESS_BAD;
	  else next_state <= BAD; -- jiny
	  end if;
   -- - - - - - - - - - - - - - - - - - - - - - -
   when OK6 =>
      if (KEY(15 downto 0) = "0000000000000000") then -- nic
          next_state <= OK6;
	  elsif (KEY(1) = '1') then -- 1
	  	  next_state <= OK7;
	  elsif (KEY(15) = '1') then -- #
	  	  next_state <= PRINT_MESS_BAD;
	  else next_state <= BAD; -- jiny
	  end if;		  
   -- - - - - - - - - - - - - - - - - - - - - - -
   when OK7 =>
      if (KEY(15 downto 0) = "0000000000000000") then -- nic
          next_state <= OK7;
	  elsif (KEY(3) = '1') then -- 3
	  	  next_state <= OK8;
	  elsif (KEY(15) = '1') then -- #
	  	  next_state <= PRINT_MESS_BAD;
	  else next_state <= BAD; -- jiny
	  end if;
   -- - - - - - - - - - - - - - - - - - - - - - -
   when OK8 =>
      if (KEY(15 downto 0) = "0000000000000000") then -- nic
          next_state <= OK7;
	  elsif (KEY(6) = '1') then -- 6
	  	  next_state <= OK9;
	  elsif (KEY(15) = '1') then -- #
	  	  next_state <= PRINT_MESS_BAD;
	  else next_state <= BAD; -- jiny
	  end if;

   -- - - - - - - - - - - - - - - - - - - - - - -
   when OK9 =>
      if (KEY(15 downto 0) = "0000000000000000") then -- nic
          next_state <= OK9;
	  elsif (KEY(15) = '1') then -- ok
	  	  next_state <= PRINT_MESS_OK;
	  else next_state <= BAD; -- jiny
	  end if;
  
   -- - - - - - - - - - - - - - - - - - - - - - -
   when PRINT_MESS_OK =>
      next_state <= PRINT_MESS_OK;
      if (CNT_OF = '1') then
         next_state <= FINISH;
      end if;
   -- - - - - - - - - - - - - - - - - - - - - - -
   when PRINT_MESS_BAD =>
      next_state <= PRINT_MESS_BAD;
      if (CNT_OF = '1') then
         next_state <= FINISH;
      end if;

	  -- - - - - - - - - - - - - - - - - - - - - - -
   when FINISH =>
      next_state <= FINISH;
      if (KEY(15) = '1') then -- #
         next_state <= IDLE;
      end if;
   -- - - - - - - - - - - - - - - - - - - - - - -
   WHEN BAD =>
   	  next_state <= bad;
	  if (KEY(15) = '1') then -- #
	  next_state <= PRINT_MESS_BAD;
      end if;
   -- - - - - - - - - - - - - - - - - - - - - - -
   when others =>
      next_state <= IDLE;
   end case;
end process next_state_logic;

-- -------------------------------------------------------
output_logic : process(present_state, KEY)
begin
   FSM_CNT_CE     <= '0';
   FSM_MX_MEM     <= '0';
   FSM_MX_LCD     <= '0';
   FSM_LCD_WR     <= '0';
   FSM_LCD_CLR    <= '0';

   case (present_state) is
   -- - - - - - - - - - - - - - - - - - - - - - -
   when IDLE => 
   	   if (KEY(14 downto 0) /= "000000000000000") then
   	      FSM_MX_LCD     <= '0';
	      FSM_LCD_WR     <= '1';
	   end if;
   when OK1 =>
   	   if (KEY(14 downto 0) /= "000000000000000") then
   	      FSM_MX_LCD     <= '0';
	      FSM_LCD_WR     <= '1';
	   end if;
   when OK2 =>
   	   if (KEY(14 downto 0) /= "000000000000000") then
   	      FSM_MX_LCD     <= '0';
	      FSM_LCD_WR     <= '1';
	   end if;
   when OK3 =>
   	   if (KEY(14 downto 0) /= "000000000000000") then
   	      FSM_MX_LCD     <= '0';
	      FSM_LCD_WR     <= '1';
	   end if;
   when OK4 =>
   	   if (KEY(14 downto 0) /= "000000000000000") then
   	      FSM_MX_LCD     <= '0';
	      FSM_LCD_WR     <= '1';
	   end if;
   when OK5 =>
   	   if (KEY(14 downto 0) /= "000000000000000") then
   	      FSM_MX_LCD     <= '0';
	      FSM_LCD_WR     <= '1';
	   end if;
   when OK6 =>
   	   if (KEY(14 downto 0) /= "000000000000000") then
   	      FSM_MX_LCD     <= '0';
	      FSM_LCD_WR     <= '1';
	   end if;
   when OK7 =>
   	   if (KEY(14 downto 0) /= "000000000000000") then
   	      FSM_MX_LCD     <= '0';
	      FSM_LCD_WR     <= '1';
	   end if;
   when OK8 =>
   	   if (KEY(14 downto 0) /= "000000000000000") then
   	      FSM_MX_LCD     <= '0';
	      FSM_LCD_WR     <= '1';
	   end if;
   when OK9 =>
   if (KEY(14 downto 0) /= "000000000000000") then
         FSM_LCD_WR     <= '1';
      end if;
      if (KEY(15) = '1') then
         FSM_LCD_CLR    <= '1';
      end if;
   -- - - - - - - - - - - - - - - - - - - - - - -
   when PRINT_MESS_OK =>
      FSM_MX_MEM     <= '1';
      FSM_CNT_CE     <= '1';
      FSM_MX_LCD     <= '1';
      FSM_LCD_WR     <= '1';
   when PRINT_MESS_BAD =>
      FSM_MX_MEM     <= '0';
      FSM_CNT_CE     <= '1';
      FSM_MX_LCD     <= '1';
      FSM_LCD_WR     <= '1';
   -- - - - - - - - - - - - - - - - - - - - - - -
   when FINISH =>
      if (KEY(15) = '1') then
         FSM_LCD_CLR    <= '1';
      end if;
   -- - - - - - - - - - - - - - - - - - - - - - -
   when others =>
   end case;
end process output_logic;

end architecture behavioral;
